package org.greenstone.gatherer.gui;
/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.util.Utility;
/** The menu bar for the Gatherer main GUI. 
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.2
 */
public class MenuBar
    extends JMenuBar {
    /** The icon to be displayed alongside the context choosen help file. */
    private int current_tab          = -1;
    private JMenu file               = null;
    private JMenu edit               = null;
    private JMenu metadata           = null;
    private JMenu tools              = null;
    private JMenu log                = null;
    private JMenu window             = null;
    private JMenu source             = null;
    private JMenu level              = null; 
    public JMenu help                = null;
    public JMenuItem file_associations;
    public JMenuItem file_close      = null;
    public JMenuItem file_delete     = null;
    public JMenuItem file_exit       = null;
    public JMenuItem file_new        = null;
    public JMenuItem file_open       = null;
    public JMenuItem file_options    = null;
    public JMenuItem file_save       = null;
    public JMenuItem file_save_as;
    public JMenuItem edit_copy;
    public JMenuItem edit_cut;
    public JMenuItem edit_paste;
    public JMenuItem edit_undo       = null;
    public JMenuItem edit_redo       = null;
    public JMenuItem edit_replace = null;
    public JMenuItem edit_search = null;
    public JMenuItem metadata_add    = null;
    public JMenuItem metadata_edit   = null;
    public JMenuItem metadata_export = null;
    public JMenuItem tools_log       = null;
    public JMenuItem tools_size      = null;
    public JMenuItem help_about;
    public JMenuItem help_browse;
    public JMenuItem help_build;
    public JMenuItem help_collect;
    public JMenuItem help_design;
    public JMenuItem help_export;
    public JMenuItem help_general;
    public JMenuItem help_metaedit;
    public JMenuItem help_mirror ;
    public JMenuItem help_preview;
    public MagicMenuItem metadata_view   = null;

    public MenuBar(MenuListener menu_listener) {

	file = new JMenu(get("Menu.File"));
	file.setMnemonic(KeyEvent.VK_F);
	this.add(file);

	file_associations = new JMenuItem(get("Menu.File_Associations"), KeyEvent.VK_A);
	file_associations.addActionListener(Gatherer.g_man);	  

	file_close = new JMenuItem(get("Menu.File_Close"), KeyEvent.VK_C);
	file_close.addActionListener(Gatherer.g_man);
	file_close.setEnabled(false);

	file_delete = new JMenuItem(get("Menu.File_Delete"), KeyEvent.VK_D);
	file_delete.addActionListener(Gatherer.g_man);

	file_exit = new JMenuItem(get("Menu.File_Exit"), KeyEvent.VK_X);
	file_exit.addActionListener(Gatherer.g_man);

	file_new = new JMenuItem(get("Menu.File_New"), KeyEvent.VK_N);
	file_new.addActionListener(Gatherer.g_man);

	file_open = new JMenuItem(get("Menu.File_Open"), KeyEvent.VK_O);
	file_open.addActionListener(Gatherer.g_man);

	file_options = new JMenuItem(get("Menu.File_Options"), KeyEvent.VK_P);
	file_options.addActionListener(Gatherer.g_man);	  

	file_save = new JMenuItem(get("Menu.File_Save"), KeyEvent.VK_S);
	file_save.addActionListener(Gatherer.g_man);
	file_save.setEnabled(false);

	file_save_as = new JMenuItem(get("Menu.File_Save_As"), KeyEvent.VK_A);
	//file_save_as.addActionListener(Gatherer.g_man);
	file_save_as.setEnabled(false);		  

	// Layout file menu
	file.add(file_new);
	file.add(file_open);
	file.add(file_save);
	//file.add(file_save_as);
	file.add(file_close);
	//file.add(new JSeparator());
	file.add(file_delete);
	file.add(new JSeparator());
	file.add(file_associations);
	file.add(file_options);
	file.add(new JSeparator());
	file.add(file_exit);

	// Edit menu
	edit = new JMenu(get("Menu.Edit"));
	edit.setMnemonic(KeyEvent.VK_E);
	this.add(Box.createHorizontalStrut(15));
	this.add(edit);
		  
	edit_cut = new JMenuItem(get("Menu.Edit_Cut") + " (ctrl-x)", KeyEvent.VK_X);
	edit_cut.addActionListener(Gatherer.g_man);

	edit_copy = new JMenuItem(get("Menu.Edit_Copy") + " (ctrl-c)", KeyEvent.VK_C);
	edit_copy.addActionListener(Gatherer.g_man);

	edit_paste = new JMenuItem(get("Menu.Edit_Paste") + " (ctrl-v)", KeyEvent.VK_V);
	edit_paste.addActionListener(Gatherer.g_man);

	edit_undo = new JMenuItem(get("Menu.Edit_Undo"), KeyEvent.VK_U);
	edit_undo.setEnabled(false);

	edit_redo = new JMenuItem(get("Menu.Edit_Redo"), KeyEvent.VK_D);
	edit_redo.setEnabled(false);

	edit_search = new JMenuItem(get("Menu.Edit_Search"), KeyEvent.VK_S);
	edit_search.addActionListener(Gatherer.g_man);

	edit_replace = new JMenuItem(get("Menu.Edit_Replace"), KeyEvent.VK_R);
	edit_replace.addActionListener(Gatherer.g_man);

	edit.add(edit_cut);
	edit.add(edit_copy);
	edit.add(edit_paste);
	//edit.add(new JSeparator());
	//edit.add(edit_undo);
	//edit.add(edit_redo);
	//edit.add(new JSeparator());
	//edit.add(edit_search);
	//edit.add(edit_replace);

	// Metadata menu
	metadata = new JMenu(get("Menu.Metadata"));
	metadata.setEnabled(false);
	metadata.setMnemonic(KeyEvent.VK_M);

	metadata_add = new JMenuItem(get("Menu.Metadata_Add"), KeyEvent.VK_I);
	metadata_add.addActionListener(Gatherer.g_man);

	metadata_edit = new JMenuItem(get("Menu.Metadata_Edit"), KeyEvent.VK_E);
	metadata_edit.addActionListener(Gatherer.g_man);

	metadata_export = new JMenuItem(get("Menu.Metadata_Export"), KeyEvent.VK_X);
	metadata_export.addActionListener(Gatherer.g_man);

	metadata_view = new MagicMenuItem(get("Menu.Metadata_View") + " " + get("FileActions.No_Selection"), KeyEvent.VK_A);
	metadata_view.addActionListener(Gatherer.g_man);	

	metadata.add(metadata_add);
	metadata.add(metadata_edit);
	metadata.add(metadata_export);
	metadata.add(metadata_view);
	this.add(Box.createHorizontalStrut(15));
	this.add(metadata);

	// Tools menu
	tools = new JMenu(get("Menu.Tools"));

	tools_log = new JMenuItem(get("Menu.Tools_Log"), KeyEvent.VK_L);
	tools_log.addActionListener(Gatherer.g_man);

	tools_size = new JMenuItem("Calculate Record Size", KeyEvent.VK_S);
	tools_size.addActionListener(Gatherer.g_man);

	tools.add(tools_log);
	tools.add(tools_size);
	//this.add(Box.createHorizontalStrut(15));
	//this.add(tools);

	// Help menu
	help = new JMenu(get("Menu.Help"));
	//help.addMenuListener(menu_listener);
	help.setIcon(Utility.HELP_ICON);

	help_about = new JMenuItem(get("Menu.Help_About"));
	help_about.addActionListener(Gatherer.g_man);

	help_browse = new JMenuItem(get("GUI.Browser"), Utility.BLANK_ICON);
	help_browse.addActionListener(Gatherer.g_man);

	help_build = new JMenuItem(get("GUI.Create"), Utility.BLANK_ICON);
	help_build.addActionListener(Gatherer.g_man);

	help_collect = new JMenuItem(get("GUI.Collection"), Utility.BLANK_ICON);
	help_collect.addActionListener(Gatherer.g_man);

	help_design = new JMenuItem(get("GUI.Build"), Utility.BLANK_ICON);
	help_design.addActionListener(Gatherer.g_man);

	help_export = new JMenuItem(get("GUI.Export"), Utility.BLANK_ICON);
	help_export.addActionListener(Gatherer.g_man);

	help_general = new JMenuItem(get("Source.General"));
	help_general.addActionListener(Gatherer.g_man);

	help_metaedit = new JMenuItem(get("GUI.MetaEdit"), Utility.BLANK_ICON);
	help_metaedit.addActionListener(Gatherer.g_man);

	help_mirror = new JMenuItem(get("GUI.Mirroring"), Utility.BLANK_ICON);
	help_mirror.addActionListener(Gatherer.g_man);

	help_preview = new JMenuItem(get("GUI.Preview"), Utility.BLANK_ICON);
	help_preview.addActionListener(Gatherer.g_man);

	// Layout help
	help.add(help_general);
	help.add(new JSeparator());
	if(Gatherer.config.get("workflow.browse", true)) {
	    help.add(help_browse);
	}
	if(Gatherer.config.get("workflow.mirror", true)) {
	    help.add(help_mirror);
	}
	if(Gatherer.config.get("workflow.gather", true)) {
	    help.add(help_collect);
	}
	if(Gatherer.config.get("workflow.enrich", true)) {
	    help.add(help_metaedit);
	}
	if(Gatherer.config.get("workflow.design", true)) {
	    help.add(help_design);
	}
	if(Gatherer.config.get("workflow.export", true)) {
	    help.add(help_export);
	}
	if(Gatherer.config.get("workflow.create", true)) {
	    help.add(help_build);
	}
	if(Gatherer.config.get("workflow.preview", true)) {
	    help.add(help_preview);
	}
	help.add(new JSeparator());
	help.add(help_about);
	this.add(Box.createHorizontalGlue());
	this.add(help);
    }

    public void collectionChanged(boolean ready) {
	file_close.setEnabled(ready);
	file_save.setEnabled(ready);
	file_save_as.setEnabled(ready);
	metadata.setEnabled(ready);
	tools.setEnabled(ready);
	if(ready) {
	    Gatherer.c_man.undo.registerRedoSource(edit_redo);
	    Gatherer.c_man.undo.registerUndoSource(edit_undo);
	}
    }

    /** Once a quit has been requested by the user, prevent any further menu selections. */
    public void exit() {
	edit.setEnabled(false);
	file.setEnabled(false);
	help.setEnabled(false);
	metadata.setEnabled(false);
    }

    public void setMetaAuditSuffix(String metaaudit_suffix) {
	///ystem.err.println("**** Set suffix: " + metaaudit_suffix);
	if(metaaudit_suffix == null) {
	    metadata_view.setText(get("Menu.Metadata_View") + " " + get("FileActions.No_Selection"));
	    metadata_view.setEnabled(false);
	}
	else {
	    ///ystem.err.println("Set metadata view suffix: " + metaaudit_suffix);
	    metadata_view.setText(get("Menu.Metadata_View") + " " + metaaudit_suffix);
	    metadata_view.setEnabled(true);
	}
    }

    /** Set the enabled state of one of the help menu items, based on its 'tabs' current state. Note that this method should only be called from the AWTEvent thread. */
    public void tabEnabled(int tab_index, boolean state) {
	JMenuItem selected = help.getItem(tab_index + 2); // Remember general and separator items
	selected.setEnabled(state);
    }

    /** In order to provide context aware help advice we keep track of which
	  *  tab the user has open, and then highlight that help menu item with
	  *  separators.
	  *  @param tab_index The index of the selected tab (0-7).
	  */
    public void tabSelected(int tab_index) {
	JMenuItem selected;
	if(current_tab != -1) {
				// Remove the image
	    selected = help.getItem(current_tab);
	    if(selected != null) {
		selected.setIcon(Utility.BLANK_ICON);
	    }
	}
	current_tab = tab_index + 2;
	selected = help.getItem(current_tab);
	if(selected != null) {
	    selected.setIcon(Utility.HELP_ICON);
	}
	selected = null;
    }
	 
    /** Retrieves a key from the Dictionary, using no extra arguments.
	  * @param key A String which maps to a certain phrase from the Dictionary.
	  * @return The String the matches the key or an error String if no match
	  * found.
	  */
    private String get(String key) {
	return get(key, null);
    }

    /** Retrieves a key from the Dictionary, providing extra arguments to
	  * be inserted using a String array.
	  * @param key A String which maps to a certain phrase from the Dictionary.
	  * @param args A String array containing further arguments (such as
	  * formatting instructions and variable values) to be taken into account
	  * when Dictionary creates the return String.
	  * @return The String the matches the key or an error String if no match
	  * found.
	  */
    private String get(String key, String args[]) {
	if(key.indexOf('.') == -1) {
	    key = "Menu." + key;
	}
	return Gatherer.dictionary.get(key, args);
    }

    public class MagicMenuItem
	extends JMenuItem {
	
	private boolean can_enable;
	private boolean should_enable;

	public MagicMenuItem(String title, int key_event) {
	    super(title, key_event);
	    super.setEnabled(false);
	    can_enable = false;
	    should_enable = false;
	}

	public void setCanEnable(boolean can_enable) {
	    ///ystem.err.println("can_enable = " + can_enable);
	    this.can_enable = can_enable;
	    if (can_enable) {
		///ystem.err.println("setEnable(" + should_enable + ")");
		super.setEnabled(should_enable);
	    }
	    else {
		super.setEnabled(false);
	    }
	}

	public void setEnabled(boolean should_enable) {
	    ///ystem.err.println("should_enable = " + should_enable);
	    this.should_enable = should_enable;
	    if(can_enable) {
		///ystem.err.println("setEnable(" + should_enable + ")");
		super.setEnabled(should_enable);
	    }
	}
    }

    private class NonFocusMenu
	extends JMenu {
        public NonFocusMenu(String name) {
	    super(name);
            setFocusPainted(false);
        }
        
        public boolean isFocusable(){
            return false;
        }
        
        public void requestFocus(){
	}

        public boolean requestFocus(boolean t){
	    return false;
	}

	public void setFocusable(boolean f) {
	}
    }

    private class NonFocusMenuItem
	extends JMenuItem {
        public NonFocusMenuItem(String name, int key) {
	    super(name, key);
            setFocusPainted(false);
        }
        
        public boolean isFocusable(){
            return false;
        }
        
        public void requestFocus(){
	}

        public boolean requestFocus(boolean t){
	    return false;
	}

	public void setFocusable(boolean f) {
	}
		  
    }
}
