package org.greenstone.gatherer.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.checklist.CheckList;
import org.greenstone.gatherer.checklist.Entry;
import org.greenstone.gatherer.gui.GUIManager;
import org.greenstone.gatherer.gui.SimpleMenuBar;
import org.greenstone.gatherer.gui.ModalDialog;
import org.greenstone.gatherer.cdm.ElementWrapper;
import org.greenstone.gatherer.msm.MetadataSet;
import org.greenstone.gatherer.msm.MetadataSetManager;
import org.greenstone.gatherer.util.StaticStrings;
import org.greenstone.gatherer.util.Utility;
import org.w3c.dom.*;

public class NewCollectionMetadataPrompt
    extends ModalDialog {

    private boolean cancelled = true;
    private CheckList sets_list;
    private JDialog self;
    private JList elements_list;
    static private Dimension size = new Dimension(700, 380);

    public NewCollectionMetadataPrompt() {
	super(Gatherer.g_man, true);
	this.self = this;
	setJMenuBar(new SimpleMenuBar("2.1"));
	setModal(true);
	setSize(size);
	setTitle(get("NewCollectionPrompt.Title"));

	// And the remaining metadata sets.
	ArrayList sets = new ArrayList();
	// Determine what collections are available.
	File metadata_directory = new File(Utility.METADATA_DIR);
	File[] possible_mdses = metadata_directory.listFiles();
	for(int i = 0; i < possible_mdses.length; i++) {
	    String name = possible_mdses[i].getName();
	    if(name.endsWith(StaticStrings.METADATA_SET_EXTENSION) && !name.equals(Utility.EXTRACTED_METADATA_NAMESPACE + StaticStrings.METADATA_SET_EXTENSION)) {
		sets.add(new MetadataSet(possible_mdses[i]));
	    }
	}

	// Creation
	JPanel content_pane = (JPanel) getContentPane();

	JPanel instructions_panel = new JPanel();
	JLabel instructions_label1 = new JLabel(get("NewCollectionPrompt.Metadata_Instructions1"));
	JLabel instructions_label2 = new JLabel(get("NewCollectionPrompt.Metadata_Instructions2"));

	JPanel center_pane = new JPanel();

	JPanel sets_list_pane = new JPanel();
	JLabel sets_list_label = new JLabel(get("NewCollectionPrompt.Select_MDS"));
	sets_list_label.setOpaque(false);
	sets_list = new CheckList(sets, true);

	JPanel elements_list_pane = new JPanel();
	JLabel elements_list_label = new JLabel(get("NewCollectionPrompt.Metadata_Elements"));
	elements_list = new JList();
	elements_list.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	elements_list.setForeground(Gatherer.config.getColor("coloring.collection_tree_foreground", false));
	elements_list.setSelectionBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	elements_list.setSelectionForeground(Gatherer.config.getColor("coloring.collection_tree_foreground", false));

	JPanel button_pane = new JPanel();
	JButton ok_button = new JButton(get("General.OK"));
	JButton cancel_button = new JButton(get("General.Cancel"));
	// Connection
	ok_button.addActionListener(new OKButtonListener());
	cancel_button.addActionListener(new CancelButtonListener());
	sets_list.addListSelectionListener(new MetadataListSelectionListener());
	// Display
	instructions_panel.setLayout(new GridLayout(2,1));
	instructions_panel.add(instructions_label1);
	instructions_panel.add(instructions_label2);

	sets_list_pane.setLayout(new BorderLayout());
	sets_list_pane.add(sets_list_label, BorderLayout.NORTH);
	sets_list_pane.add(new JScrollPane(sets_list), BorderLayout.CENTER);

	elements_list_pane.setLayout(new BorderLayout());
	elements_list_pane.add(elements_list_label, BorderLayout.NORTH);
	elements_list_pane.add(new JScrollPane(elements_list), BorderLayout.CENTER);

	center_pane.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	center_pane.setLayout(new GridLayout(2,1,0,5));
	center_pane.add(sets_list_pane);
	center_pane.add(elements_list_pane);

	button_pane.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	button_pane.setLayout(new GridLayout(1,2,5,0));
	button_pane.add(ok_button);
	button_pane.add(cancel_button);

	content_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	content_pane.setLayout(new BorderLayout());
	content_pane.add(instructions_panel, BorderLayout.NORTH);
	content_pane.add(center_pane, BorderLayout.CENTER);
	content_pane.add(button_pane, BorderLayout.SOUTH);
	// Show
	Dimension screen_size = Gatherer.config.screen_size;
	setLocation((screen_size.width - size.width) / 2, (screen_size.height - size.height) / 2);
	setVisible(true);
    }

    public ArrayList getSets() {
	return sets_list.getSelected();
    }

    public boolean isCancelled() {
	return cancelled;
    }

    /** Gets a phrase from the dictionary based on key.
     * @param key A <strong>String</strong> which serves as the unique identifier of a phrase.
     * @return The desired phrase as a <strong>String</strong> or at least a meaningful error message.
     */
    private String get(String key) {
	if(key.indexOf(".") == -1) {
	    key = "NewCollectionPrompt." + key;
	}
	return Gatherer.dictionary.get(key);
    }

    private class CancelButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    cancelled = true;
	    self.dispose();
	}
    }

    public class MetadataListSelectionListener
	implements ListSelectionListener {
	public void valueChanged(ListSelectionEvent event) {
	    if(!sets_list.isSelectionEmpty()) {
		// Retrieve the selected set.
		Entry entry = (Entry) sets_list.getSelectedValue();
		MetadataSet set = (MetadataSet) entry.getObject();
		entry = null;
		// Build a model from its elements.
		NodeList elements = set.getElements();
		set = null;
		Vector elements_model = new Vector();
		for(int i = 0; i < elements.getLength(); i++) {
		    elements_model.add(new ElementWrapper(elements.item(i)));
		}
		elements = null;
		Collections.sort(elements_model);
		elements_list.setListData(elements_model);
		elements_model = null;
	    }
	    else {
		elements_list.setListData(new String[0]);
	    }
	}
    }

    public class OKButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
				// See if the user has selected no metadata sets, and if so confirm thats what they really want.
	    ArrayList selected_sets = sets_list.getSelected();
	    cancelled = false;
	    if(selected_sets.size() == 0) {
		WarningDialog dialog = new WarningDialog("warning.NoMetadataSetsSelected", true);
		if(dialog.display() == JOptionPane.OK_OPTION) {
		    // Otherwise we are free to go
		    self.dispose();
		}
	    }
	    else {
		self.dispose();
	    }
	}
    }
}
