/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

 




package org.greenstone.gatherer.gui.table;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.gui.table.GTableModel;
import org.greenstone.gatherer.msm.ElementWrapper;
import org.greenstone.gatherer.msm.Metadata;
import org.greenstone.gatherer.util.Utility;

public class TableCellRenderer
    extends DefaultTableCellRenderer {

    private GTableModel model = null;

    public TableCellRenderer(GTableModel model) {
	super();
	this.model = model;
    }

    /** Returns the default table cell renderer.
     * @param table The <strong>JTable</strong>.
     * @param value The value to assign to the cell at [row, column] as an <strong>Object</strong>.
     * @param isSelected <i>true</i> if cell is selected.
     * @param row The row of the cell to render as an <i>int</i>.
     * @param column The column of the cell to render as an <i>int</i>.
     * @param hasFocus <i>true</i> if cell has focus, render cell appropriately. For example, put a special border on the cell, if the cell can be edited, render in the color used to indicate editing.
     * @return The default table cell renderer <strong>Component</strong>.
     */
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
	JLabel renderer = null;
	if(value instanceof File) {
	    if(value != null) {
		// if(isSelected && hasFocus) {
		// Metadata data = model.getMetadataAtRow(row);
		// Gatherer.g_man.metaedit_pane.setSelection((File)value);
		// Gatherer.g_man.metaedit_pane.setSelectedMetadata(data);
		// }
		renderer = new JLabel(Utility.getImage("upfolder.gif"));
	    }
	}
	if(renderer == null) {
	    renderer = (JLabel) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
	}
	// Set up the component
	renderer.setOpaque(true);
	// Foreground
	if(model.isCommon(row)) {
	    renderer.setForeground(Color.black);
	}
	else {
	    renderer.setForeground(Color.gray);
	}
	// Background
	if(column <= 1) {
	    if(isSelected) {
		renderer.setBackground(Gatherer.config.getColor("coloring.workspace_heading_background", true));
	    }
	    else {
		renderer.setBackground(Gatherer.config.getColor("coloring.collection_heading_background", true));
	    }
	}
	else {
	    if(isSelected) {
		renderer.setBackground(Gatherer.config.getColor("coloring.workspace_heading_background", true));
	    }
	    else {
		renderer.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", true));
	    }
	}
	// Finally the 3rd column of cells never paint focus.
	if(column == 2) {
	    renderer.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
	}

	// We set a tooltip over the element column containing the description of the element
	if(value instanceof ElementWrapper) {
	    renderer.setToolTipText(((ElementWrapper)value).getToolTip());
	}	

	return renderer;
    }
}
