package org.greenstone.gsdl3.service;

// Greenstone classes
import org.greenstone.gdbm.*;
import org.greenstone.gsdl3.util.*;

// XML classes
import org.w3c.dom.Element; 
import org.w3c.dom.Document;
import org.w3c.dom.NodeList; 

import java.util.HashMap;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.*; //Document;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Hits;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.index.Term;

import java.io.File;
/**
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 5257 $
 */

public class LuceneSearch 
    extends ServiceRack {

    // the services on offer
    // these strings must match what is found in the properties file
    protected static final String TEXT_QUERY_SERVICE = "TextQuery";
    
    protected static final String QUERY_PARAM = "query";
    protected static final String WORK_PARAM = "work";
    
    public boolean configure(Element info, Element extra_info) {
	
	Element tq_service = this.doc.createElement(GSXML.SERVICE_ELEM);
	tq_service.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_QUERY);
	tq_service.setAttribute(GSXML.NAME_ATT, TEXT_QUERY_SERVICE);
	this.short_service_info.appendChild(tq_service);
	
	return true;
	
    }

    protected Element getServiceDescription(String service, String lang, String subset) {
	
	if (!service.equals(TEXT_QUERY_SERVICE)) {
	    return null;
	}
	Element tq_service = this.doc.createElement(GSXML.SERVICE_ELEM);
	tq_service.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_QUERY);
	tq_service.setAttribute(GSXML.NAME_ATT, TEXT_QUERY_SERVICE);
	return tq_service;
    }
    
    /** Process a text query - implemented by concrete subclasses */
    protected Element processTextQuery(Element request) {

	// Create a new (empty) result message
	Element result = this.doc.createElement(GSXML.RESPONSE_ELEM);
	result.setAttribute(GSXML.FROM_ATT, TEXT_QUERY_SERVICE);
	result.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_PROCESS);
	Element doc_node_list = this.doc.createElement(GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	result.appendChild(doc_node_list);
	
	// Get the parameters of the request
	Element param_list = (Element) GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	if (param_list == null) {
	    System.err.println("Error: TextQuery request had no paramList.");
	    return result;  // Return the empty result
	}

	// Process the request parameters
	HashMap params = GSXML.extractParams(param_list, false);

	// Make sure a query has been specified
	String query_string = (String) params.get(QUERY_PARAM);
	if (query_string == null || query_string.equals("")) {
	    System.err.println("Error: TextQuery request had no query string.");
	    return result;  // Return the empty result
	}

        try {
	    String index_dir = GSFile.collectionIndexDir(this.site_home, this.cluster_name);
	    index_dir += File.separator+"idx";
	    Searcher searcher = new IndexSearcher(index_dir);
	    Analyzer analyzer = new StandardAnalyzer();
	    
	    Term term = new Term("content", query_string);
        
	    Query query = new TermQuery(term);


	    System.out.println("Searching for: " + query.toString("content"));

	    Hits hits = searcher.search(query);
	    System.out.println(hits.length() + " total matching documents");

	    for (int i=0; i<hits.length(); i++) {
		org.apache.lucene.document.Document luc_doc = hits.doc(i);
		String node_id = luc_doc.get("nodeID");
		Element node = this.doc.createElement(GSXML.DOC_NODE_ELEM);
		node.setAttribute(GSXML.NODE_ID_ATT, node_id);
		doc_node_list.appendChild(node);
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}
	
	return result;
	    
    }

}
